open Trs2system
open Projet


(* Application du termination checker à un ou plusieurs fichiers trs *)

let _ =
  let arg = Array.length (Sys.argv) in

  if arg < 2 then
    failwith "Utilisation de test: ./test foo.trs bar.trs ..."
  else if Sys.argv.(1) = "-help" || Sys.argv.(1) = "--help" || Sys.argv.(1) = "-h" then
    print_string "Utilisation de test: ./test foo.trs bar.trs ...\n"
  else

    for i=1 to arg-1 do

      let filename = Sys.argv.(i) in
      let n = String.length filename in
      let name = String.sub filename 0 (n-4)
      and ext = String.sub filename (n-4) 4 in

      if ext <> ".trs" then
        failwith "L'extension du fichier doit être .trs"
      else (

        print_string ("Étude du système "^name^"...\n");
        flush stdout;
        let file = open_in filename in

        (
          try
            let lexbuf = Lexing.from_channel file in
            try

              let spec = Trs_parser.spec Trs_lexer.token lexbuf in
              let s = system_of_spec spec in
              let g = Projet.main s in

              let nb_graph = List.length g in
              if nb_graph = 0 then
                print_string ("Le système de récriture "^name^" termine.\n")
              else if nb_graph = 1 then (
                print_string ("Graphe minimal à étudier pour savoir si le système "^name^" termine: "^name^".dot.\n");
                Graph.Pack.Digraph.dot_output (List.hd g).graph (name^".dot")
              ) else (
                print_string ("Graphes minimaux à étudier pour savoir si le système "^name^" termine: "^name^".0.dot à "^name^"."^(string_of_int (nb_graph-1))^".dot.\n") ;
                let count = ref (-1) in
                List.iter (fun x -> incr count; Graph.Pack.Digraph.dot_output x.graph (name^"."^(string_of_int !count)^".dot")) g
              )

            with Failure _ -> failwith ("Le fichier "^filename^" ne correspond pas à la spécification trs.")
          with Parsing.Parse_error -> failwith ("Le fichier "^filename^" ne correspond pas à la spécification trs.")
        );

        close_in file;
        flush stdout
      )

      done;;
