open Trs_ast
open Match_unif
open Projet


(* Gestion de la liste des variables *)

let var_list = ref [];;
let var = ref (-1);;

let rec vl_of_il il =
  match il with
    | [] -> []
    | id::q ->
        incr var;
        (id,!var)::(vl_of_il q);;


(* Interprétation d'un terme *)

let rec term_of_tm t =
  match t with
    | Tm(id,l) ->
        try
          let num = List.assoc id !var_list in
          Var num
        with Not_found ->
          let (_,f) = id in
          Term(f,List.map term_of_tm l);;


(* Interprétation d'une règle *)

let rule_of_rule r =
  match r with
    | Rew(_,t1,t2) -> (term_of_tm t1, term_of_tm t2)
    | RelRew(_,t1,t2) -> (term_of_tm t1, term_of_tm t2);;


(* Interprétation d'une liste de règles *)

let rec rules_of_list rules_list =
  match rules_list with
    | [] -> []
    | r::q -> (rule_of_rule r)::(rules_of_list q);;


(* Interprétation d'un déclaration *)
(* On ne tient pas compte des éléments plus sophistiqués que ceux de nos
   systèmes, comme les stratégies de réduction. *)

let rules_of_decl decl =
  match decl with
    | VarDecl id_list -> var_list := (vl_of_il id_list)@(!var_list); []
    | RulesDecl rules_list -> rules_of_list rules_list
    | _ -> [];;


(* Interprétation d'une spécification *)

let rec system_of_spec spec : system =
  match spec with
    | [] -> []
    | decl::q -> (rules_of_decl decl)@(system_of_spec q);;
